﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using IDTechSDK;

namespace L100_Simple_Demo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            IDT_L100.setCallback(MessageCallBack);
            cbBaud.SelectedIndex = 0;
        }

        private void MessageCallBack(IDTechSDK.IDT_DEVICE_Types type, DeviceState state, byte[] data, IDTTransactionData cardData, EMV_Callback emvCallback, RETURN_CODE transactionResultCode)
        {
         
            switch (state)
            {

                case DeviceState.ToConnect:
                    SetOutputText("To connect\r\n");
                    break;
                case DeviceState.DefaultDeviceTypeChange:
                    SetOutputText("DefaultDeviceTypeChange\r\n");
                    break;
                case DeviceState.Connected:
                    SetOutputText("Connected\r\n");
                    break;
                case DeviceState.Disconnected:
                    SetOutputText("Disconnected\r\n");
                    break;
                case DeviceState.ConnectionFailed:
                    SetOutputText("Connection Failed\r\n");
                    break;
                case DeviceState.Notification:
                    SetOutputText("Notification\r\n");
                    break;
                case DeviceState.TransactionData:
                    if (cardData == null) break;
                    //output parsed card data
                    SetOutputText("Return Code: " + transactionResultCode.ToString() + "\r\n");
                    if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA)
                    {
                        SetOutputText("PIN Data received:\r\nKSN: " + cardData.pin_KSN + "\r\nPINBLOCK: " + cardData.pin_pinblock + "\r\nKey Entry: " + cardData.pin_KeyEntry + "\r\n");
                        return;

                    }
                    break;
                case DeviceState.DataReceived:
                    SetOutputTextLog(" IN: " + Common.getHexStringFromBytes(data)+ "\r\n");
                    break;
                case DeviceState.DataSent:
                    SetOutputTextLog(" OUT: " + Common.getHexStringFromBytes(data)+ "\r\n");
                    break;
                case DeviceState.CommandTimeout:
                    SetOutputText("Command Timeout\r\n");
                    break;
 
                case DeviceState.MSRDecodeError:
                    SetOutputText("MSR Decode Error\r\n");
                   break;

                case DeviceState.SwipeTimeout:
                    SetOutputText("Swipe Timeout\r\n");
                    break;
                case DeviceState.TransactionCancelled:
                    System.Diagnostics.Debug.WriteLine("TransactionCancelled\r\n");
                    break;
                case DeviceState.DeviceTimeout:
                    SetOutputText("Device Timeout\r\n");
                    break;
                case DeviceState.TransactionFailed:

                    SetOutputText("Transaction Failed: " + IDTechSDK.errorCode.getErrorString(transactionResultCode) + "\r\n");
                    break;
                case DeviceState.EMVCallback:
                    SetOutputText("EMV Callback\r\n");
                    break;
                case DeviceState.PINpadKeypress:
                    SetOutputText("PINPad Key was pressed\r\n");
                    break;
                default:
                    break;

            }
        }
        delegate void SetTextCallback(string text);

        private void SetOutputText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (tbOutput.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetOutputText);
                Invoke(d, new object[] { text });
            }
            else
            {
                try { tbOutput.AppendText(text + "\r\n"); } catch (Exception ex) { }

            }
        }



        private void SetOutputTextLog(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (logOutput.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetOutputTextLog);
                Invoke(d, new object[] { text });
            }
            else
            {
                try { logOutput.AppendText(text + "\r\n"); }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine("Exception: " + ex);
                }

            }
        }

        private void rbUSB_CheckedChanged(object sender, EventArgs e)
        {
            if (rbUSB.Checked)
            {
                IDT_L100.useUSB();
                return;
            }
            if (rbCOMM.Checked)
            {
                bool isConnected = false;
                if (cbBaud.SelectedIndex == 0)
                {
                    isConnected = IDT_L100.useSerialPort(Convert.ToInt32(tbCOMPort.Text));
                }
                else
                {
                    isConnected = IDT_Device.useSerialPort(IDT_DEVICE_Types.IDT_DEVICE_L100, DEVICE_PROTOCOL_Types.DEVICE_PROTOCOL_NGA, Convert.ToInt32(tbCOMPort.Text), Convert.ToInt32(cbBaud.SelectedItem));
                }
                if (!isConnected)
                {
                    IDT_Device.setDeviceType(IDT_DEVICE_Types.IDT_DEVICE_NONE, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_UNKNOWN, DEVICE_PROTOCOL_Types.DEVICE_PROTOCOL_UNKNOWN);
                    tbOutput.AppendText("Failed to locate device Pro on specified COM port");
                    return;
                }
                IDT_Device.setDeviceType(IDT_DEVICE_Types.IDT_DEVICE_L100, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_SERIAL, DEVICE_PROTOCOL_Types.DEVICE_PROTOCOL_NGA);


            }

        }

        private void btnFirmwareVersion_Click(object sender, EventArgs e)
        {
            string firmwareVersion = "";
            RETURN_CODE rt = IDT_L100.SharedController.device_getFirmwareVersion(ref firmwareVersion);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Firmware Ver: " + firmwareVersion + "\r\n");
                System.Diagnostics.Debug.WriteLine("Firmware Ver: " + firmwareVersion);
            }
            else
            {
                tbOutput.AppendText("Get Firmware Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Firmware Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnGetPIN_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_getEncryptedPIN(1, "4111111111111111", "ENTER PIN", 45);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Enter Pin Executed Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Enter Pin Executed Successful");
            }
            else
            {
                tbOutput.AppendText("Enter Pin Executed failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Enter Pin Executed failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnSecureMessage_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_promptForKeyInput(1, 0, false, 4, 12, 30);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Prompt for Key Entry Command Successful\r\n");
            }
            else
            {
                tbOutput.AppendText("Prompt for Key Entry Command Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
            }
        }

        private void btnUnsecureMessage_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_Device.SharedController.lcd_displayMessage(1, "SmartPIN L100 Ready");
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Display Message Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Display Message Successful");
            }
            else
            {
                tbOutput.AppendText("Display Message failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Display Message failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }
    }
}
